;;CoDMER Version 2.0 December 2013

;;(C)2013 Board of Trustees of the University of Illinois
    
;;Authors: 
;;Moira Zellner, Dean Massey and Kristen Ross, University of Illinois at Chicago
;;Cristy Watkins, Field Museum
;;Lynne Westphal, US Forest Service
;;Jeremy Brooks, Ohio State University


globals[
  ;;; METRICS
  ;; Primary
  final-tick                        ;; Length of the run
  collective-position               ;; The mean individual-position of all agents  
  Spread-of-individual-positions    ;; The standard deviation of the agent individual-positions from the mean (collective position) at the end of the run
  ;; Secondary
  number-of-mutations               ;; Total number of times agents have mutated their individual-position values
  total-number-of-interactions      ;; Number of times there was an interaction
  sum-number-of-times-interacting   ;; The sum of number-of-times-interacting (a turtle variable), which is sum of the times each of the agents have interacted
  sum-flagged                       ;; The sum of the times any agent has been selected to possibly interact
  average-interactions-per-agent    ;; sum-number-of-times-interacting / Number-of-Agents
  final-decision                    ;; The numerical score that is the outcome - set based on the decision strategy used, so it is either the collective position of all the agents, the mean of the individual positions for the point persons, or one of the individual-position values of one of the point persons
  Count-inter-formal                ;; The number of intergroup formal meetings that have occurred.
  Count-inter-informal              ;; The number of intergroup informal meetings that have occurred.
  Count-intra-formal                ;; The number of intragroup formal meetings that have occurred.
  Count-intra-informal              ;; The number of intragroup informal meetings that have occurred.

  ;; USED IN PROCESSES 
  Scenario                          ;; Shortened name of the scenario set by the Scenario-Tested slider
  Total-Number-of-Agents            ;; Number of agents
  Total-Number-of-Groups            ;; Number of groups
  sample-group-here                 ;; A variable used when checking for inter-group informal interactions.
  maximum-iterations                ;; The tick on which the run ends. In this model version, 1000 is the maximum length.
  type-of-interaction               ;; the type of interaction being displayed in the landscape at that momemt.
  Frequency-of-inter-group-formal-meetings ;; Ticks between intergroup formal meetings
  ;;; Imported data lists used with informal meetings
  inter-group-informal-list         ;; Imported data used to determine intergroup informal meetings
  intra-group-informal-list         ;; Imported data used to determine intragroup informal meetings
]

breed [ point-persons point-person ]
breed [ advisors advisor ]

turtles-own [ ;; Shared by point-persons and advisors
  individual-position               ;; Represents intensity of desired action in relation to one specific issue. Ranges from 0 to 1. E.g. 0 represents take no action and 1 represents greatest possible level of action.
  position-future                   ;; Buffer layer for the new updated individual-position value. It becomes the individual-position after all agents have calcualted their own future individual-positions.
  ID                                ;; Unique identification number for turtle, corresponds to the x coordinate of the patch it is on.
  inactive-color                    ;; The color of the agent if it is not interacting. the color darkens if it is interacting. Colors are based on what group it belongs to.
  updating?                         ;; A flag to indicate the turtle was selected to interact witb other agents. If other conditions are met, it will interact.
  group                             ;; The number of the group that the agent belongs to; 0, 1, 2, etc.
  number-of-times-interacting       ;; The number of times that the turtle has interacted with other agents
]

patches-own [
  respect                         ;; How much the agent in the row respects the agent in that column. on a scale of 0 to 1, where 1 = highest level and 0 = no respect.
  normalized-respect              ;; A respect value normalized by the total respect the agent has for all other agents including itself participating in an interaction. On a scale of 0 to 1.
  patch-inactive-color            ;; The color of the patch if it is not active (involved in an interaction). The color darkens if it is interacting. Colors are based on what group the agent in that row belongs to.
]


to setup
  clear-all
  setup-scenario
  ;; Resize world so that there is space for only one agent in each row or column (e.g. with 18 agents, it will be 18 x 18 cells)
  resize-world 0 ( Total-Number-of-Agents - 1 ) ( ( Total-Number-of-Agents - 1 ) * -1 ) 0
  set maximum-iterations 1000 ;; Arbitrarily setting the maximum run lemgth to be 1,000 days, which is far longer than what would happen with any of the scenarios, but still setting a limit so the model does not just go indefinitely.

  ;; Create the agents and assign them to a place on the landscape, then assigning them groups, positions, and respects based on the input files.
  create-agents
  setup-positions
  setup-respect
  setup-meetings ;; Set up meetings based on input files.
  ;; Color the patches and turtles so the colors correspond to group memebrship
  do-colors
  reset-ticks
end

to setup-scenario
  ;; Setting the number of total agents and groups based on the scenario and then using a shortened name to refer to the scenario in the rest of the code
  if Scenario-Tested = "1a - No KL - Baseline"
  [ set Total-Number-of-Agents 6
    set Total-Number-of-Groups 2
    set Scenario "1a"
  ]
  if Scenario-Tested = "1b - With KL - high respect and high interactions"
  [ set Total-Number-of-Agents 7
    set Total-Number-of-Groups 2
    set Scenario "1b"
  ]  
  if Scenario-Tested = "1c - With KL - high respect and low interactions"
  [ set Total-Number-of-Agents 7
    set Total-Number-of-Groups 2
    set Scenario "1c"
  ]
  if Scenario-Tested = "1d- With KL - low respect and high interactions"
  [ set Total-Number-of-Agents 7
    set Total-Number-of-Groups 2
    set Scenario "1d"
  ]
  if Scenario-Tested = "2a - No KL – baseline"
  [ set Total-Number-of-Agents 19
    set Total-Number-of-Groups 4
    set Scenario "2a"
  ]  
  if Scenario-Tested = "2b - With KL - high respect and high interactions"
  [ set Total-Number-of-Agents 20
    set Total-Number-of-Groups 4
    set Scenario "2b"
  ]
  if Scenario-Tested = "2c - With KL - high respect and high interactions but KL has low respect for Subgroup 3 agents"
  [ set Total-Number-of-Agents 20
    set Total-Number-of-Groups 4
    set Scenario "2c"
  ]
  if Scenario-Tested = "2d - With KL - high respect and high interactions - Subgroup 0 and 3 same size"
  [ set Total-Number-of-Agents 28
    set Total-Number-of-Groups 4
    set Scenario "2d"
  ]  
  if Scenario-Tested = "2e - No KL - Subgroup 0 point person replaced with agent that interacts more and has higher respect"
  [ set Total-Number-of-Agents 19
    set Total-Number-of-Groups 4
    set Scenario "2e"
  ]  
end

to create-agents
  ;; Creating one turtle per row and column and placing them where the pxcor = the absolute value of the pycor. This puts them in a diagonal line from the upper left to bottom right.
  foreach sort patches with [ pxcor = abs ( pycor ) ] [
    ask ? [
      if count turtles < Total-Number-of-Agents [
        sprout 1[
          set size .6
          set ID ( abs ( pxcor ) ) ;; ID is unique to each turtle
          set breed advisors ;; All agents start as advisors then in the next step are set to be point persons if conditions are met.
          set updating? false
        ]
      ]
    ]
  ]
  
  ;; Now setting up groups and point persons
  ;; We assume that there is only one point person per group
  ;; The groups and point-person status are set based on the individual input files for each scenario
  
  ;; Case 1
  ;; Without KL
  if Scenario = "1a" [
    file-open "1-NoKLGroups.prn"
    let agent-groups file-read
    file-close
      ( foreach sort turtles agent-groups
        [ ask ?1 [
          set group ?2
        ] ] )
    ask turtle 0 [
      set breed point-persons
      set shape "square"
    ]
    ask turtle 4 [
      set breed point-persons
      set shape "square"
    ]
    ask turtles with [ shape = "default" ] [
      set breed advisors
      set shape "triangle"
    ]
  ]
  ;; With KL
  if Scenario = "1b" or Scenario = "1c" or Scenario = "1d"[
    file-open "1-KLGroups.prn"
    let agent-groups file-read
    file-close
      ( foreach sort turtles agent-groups
        [ ask ?1 [
          set group ?2
        ] ] )
    ask turtle 0 [
      set breed point-persons
      set shape "square" ;; Different shape to indicate the point person
    ]
    ask turtle 5 [
      set breed point-persons
      set shape "square" ;; Different shape to indicate the point person
    ]
    ask turtles with [ shape = "default" ] [ ;; I.E. one that are not point-persons
      set breed advisors
      set shape "triangle"
    ]
  ]
  
  ;; Case 2
  ;; No KL
  if Scenario = "2a" or Scenario = "2e" [
    file-open "2-NoKLGroups.prn"
    let agent-groups file-read
    file-close
      ( foreach sort turtles agent-groups
        [ ask ?1 [
          set group ?2
        ] ] )
    ask turtle 0 [
      set breed point-persons
      set shape "square"
    ]
    ask turtle 3 [
      set breed point-persons
      set shape "square"
    ]
    ask turtle 5 [
      set breed point-persons
      set shape "square"
    ]
    ask turtle 7 [
      set breed point-persons
      set shape "square"
    ]
    
    ask turtles with [ shape = "default" ] [
      set breed advisors
      set shape "triangle"
    ]
  ]
  ;; KL and 20 total agents
  if Scenario = "2b" or Scenario = "2c" [
    file-open "2-KL-20Groups.prn"
    let agent-groups file-read
    file-close
      ( foreach sort turtles agent-groups
        [ ask ?1 [
          set group ?2
        ] ] )
    ask turtle 0 [
      set breed point-persons
      set shape "square"
    ]
    ask turtle 4 [
      set breed point-persons
      set shape "square"
    ]
    ask turtle 6 [
      set breed point-persons
      set shape "square"
    ]
    ask turtle 8 [
      set breed point-persons
      set shape "square"
    ]
    
    ask turtles with [ shape = "default" ] [
      set breed advisors
      set shape "triangle"
    ]
  ] 
  
  if Scenario = "2d" [
    file-open "2-KL-28Groups.prn"
    let agent-groups file-read
    file-close
      ( foreach sort turtles agent-groups
        [ ask ?1 [
          set group ?2
        ] ] )
    ask turtle 0 [
      set breed point-persons
      set shape "square"
    ]
    ask turtle 12 [
      set breed point-persons
      set shape "square"
    ]
    ask turtle 14 [
      set breed point-persons
      set shape "square"
    ]
    ask turtle 16 [
      set breed point-persons
      set shape "square"
    ]
    
    ask turtles with [ shape = "default" ] [
      set breed advisors
      set shape "triangle"
    ]
  ] 
  
  ;; Assigning colors to the agents based on group membership. these are the colors for when the agents are not actively interacting.
  ask turtles [
    if group = 0 [ set inactive-color 97 ] ;; blue (sky)
    if group = 1 [ set inactive-color 127 ] ;; purple (magenta)
    if group = 2 [ set inactive-color 27 ] ;; orange
    if group = 3 [ set inactive-color 47 ] ;; light yellow
  ]
  
end

to setup-positions
;;this procedure assigns individual-position values to agents based on the input files
  ;; Case 1
  if Scenario = "1a" [ file-open "1a-NoKL-Positions.prn" ]
  if Scenario = "1b" or Scenario = "1c" or Scenario = "1d" [ file-open "1-KL-Positions.prn" ]
  ;; Case 2
  if Scenario = "2a" [ file-open "2a-NoKL-Positions.prn" ]
  if Scenario = "2b" or Scenario = "2c" [ file-open "2-KL-Positions.prn" ]
  if Scenario = "2d" [ file-open "2d-KL-28-Positions.prn" ]
  if Scenario = "2e" [ file-open "2e-NB-Positions.prn" ]
    let agent-positions file-read
    file-close
      ( foreach sort turtles agent-positions
        [ ask ?1 [
          set individual-position ?2
        ] ] ) 

  ;; Saving the collective individual-position value
  set collective-position mean [ individual-position ] of turtles
  
  ;; Setting up the display of labels for the agents
  ask turtles [
    set label-color magenta
    set label precision individual-position 2
  ]
end

to setup-respect
  ;;This procedure assigns the respect values that agents have for themselves and others. Values are taken from input files,
  
  ;; Case 1
  ;; No KL
  if Scenario = "1a" [ file-open "1a-Respect.prn" ]
  ;; with KL
  if Scenario = "1b" or Scenario = "1c" or Scenario = "1d" [ file-open "1-KL-Respect.prn" ]
  ;; Case 2
  ;; No KL
  if Scenario = "2a" [ file-open "2-NoKL-Respect.prn" ]
  if Scenario = "2b" or Scenario = "2c" [ file-open "2-KL-Respect.prn" ]
  if Scenario = "2d" [ file-open "2d-KL-28-Respect.prn" ]
  if Scenario = "2e" [ file-open "2e-NB-Respect.prn" ]
  let patch-respect file-read
  file-close
    ( foreach sort patches patch-respect
      [ ask ?1 [
        set respect ?2
      ] ] ) 

  if Scenario = "1d" [ ;; Same as 1a but low mutual respect between KL and all other agents
    ask patches with [ pycor = -4 or pxcor = 4 ] [ set respect .1 ]
    ask patch 4 -4 [ set respect 1 ]
  ]

  if Scenario = "2c" [ ;; Same as 2b but KL has low respect for those in the volunteer group 
    ask patches with [ pycor = -3 and pxcor > 3 ] [ set respect .01 ]
  ]
    
  
  ask patches [
    ;; setting up the labels and display of the patches
    set plabel-color magenta
    set plabel precision respect 3
    ;; the next line tells the patch what color it should be based on the group the agent in its row is in. The patch will have a slightly lighter color than the agents.
    set patch-inactive-color ( sum [ inactive-color ] of turtles with [ id = [ abs pycor ] of myself ] ) + 2
  ]
  
end

to setup-meetings
  ;; Setting up meetings based on input files
  
  ;; Informal Meetings
  ;; Intergroup Informal
  ;; Case 1
  if Scenario = "1a" [ file-open "1a-InterGroupInformal.prn" ]
  if Scenario = "1b" or Scenario = "1d" [ file-open "1-KL-InterGroupInformal.prn" ]
  if Scenario = "1c" [ file-open "1c-KL-InterGroupInformal.prn" ]
  ;; Case 2
  if Scenario = "2a" [ file-open "2a-InterGroupInformal.prn" ]
  if Scenario = "2b" or Scenario = "2c" [ file-open "2-KL-InterGroupInformal.prn" ]
  if Scenario = "2d" [ file-open "2d-InterGroupInformal.prn" ]
  if Scenario = "2e" [ file-open "2e-NB-InterGroupInformal.prn" ]
  
  set inter-group-informal-list file-read
  file-close
  
  ;; Intragroup Informal
  ;; Case 1
  if Scenario = "1a" [ file-open "1a-IntraInformal.prn" ]
  if Scenario = "1b" or Scenario = "1d" [ file-open "1-KL-IntraInformal.prn" ]
  if Scenario = "1c" [ file-open "1c-KL-IntraInformal.prn" ]
  ;; Case 2
  if Scenario = "2a" [ file-open "2a-IntraInformal.prn" ]
  if Scenario = "2b" or Scenario = "2c" [ file-open "2-KL-IntraInformal.prn" ]
  if Scenario = "2d" [ file-open "2d-IntraInformal.prn" ]
  if Scenario = "2e" [ file-open "2e-NB-IntraInformal.prn" ]

  set intra-group-informal-list file-read
  file-close
  
  ;; Formal inter-group meeting frequency
  if Scenario = "1a" or Scenario = "1b" or Scenario = "1c" or Scenario = "1d" [ set Frequency-of-inter-group-formal-meetings 20 ] ;; Every 20 days in case 1
  if Scenario = "2a" or Scenario = "2b" or Scenario = "2c" or Scenario = "2d" or Scenario = "2e" [ set Frequency-of-inter-group-formal-meetings 0 ] ;; Never in case 2
end

to do-colors
  ;; Color the patches and agents
  ;; Agents and patches not interacting have a lighter color
  ask turtles with [ updating? = true ] [ set color inactive-color - 2]
  ask turtles with [ updating? = false ] [ set color inactive-color ]
  
  ;; patches only have a normalized-respect score not equal to 0 if they are interacting
  ask patches with [ normalized-respect != 0 ] [ set pcolor patch-inactive-color - 2 ]
  ask patches with [ normalized-respect = 0 ] [ set pcolor patch-inactive-color ]
end

to go
   ;; First, checking for whole-group convergence
  ;; See if the spread of individual positions is less tha or equal to .01
  if ( abs ( max [ individual-position ] of turtles - min [ individual-position ] of turtles ) ) <= .01 [
    ;; If convergence has happened, then record the current tick and stop the run
    set final-tick ticks
    stop
  ]
  
  ;; Second, if the time limit of the run is up, then make a decision and stop the run
  if ticks = maximum-iterations [
    make-decision
    stop
  ]
  
  ;; Mechanisms
  
  ;; Check to see if there will be interactions, and if yes, update individual-positions
  Check-for-interaction
  ;; Update graphs and global variables
  do-plots-and-globals
  tick
end

to make-decision
  ;; This procedure is a holdover from the previos model version, but in the specific cases being modeled, convergence would have happened by now.
  ;; In these scenarios, this procedure is only accessed if the run goes on for 1,000 ticks. The procedure ensures that the run does not go on indefinitely.
  ;; This procedure has the decision move up to the point persons. They meet and then if their positions are close enough, the decision is their mean preference.
  ;; If not, then one of the point persons' positions is chosen ar random.

  ;; Point persons interact.
  flag-inter-group-formal
  
  ;; If the difference between the minimum and maximum individual-positions of the point persons is within the Point-person-position-difference, then the decision is their mean position
  ifelse ( abs ( max [ individual-position ] of point-persons - min [ individual-position ] of point-persons ) ) <= .2 [
    set final-decision mean [ individual-position ] of point-persons
  ]
  ;; If not, then the position is one of the point persons positions chosen randomly.
  [
    let coin-flip random Total-Number-of-Groups
    ask one-of point-persons with [ group = coin-flip ] [
      set final-decision individual-position
    ]
  ]
  set final-tick ticks    
end

to Check-for-interaction
  ;; Each iteration, the model checks for each of the four types of interactions
  
  ;; Inter-group interactions
  
  ;; These only happen if there are more than 1 groups
  if Total-Number-of-Groups > 1 [
    
    ;; Intergroup Formal
    ;; Interaction of all point persons
    ;; If the tick is a multiple of the frequency of this type of meeting (or the first iteration), then the formal meeting happens
    if Frequency-of-inter-group-formal-meetings != 0 [
      if ticks mod Frequency-of-inter-group-formal-meetings = 0 [
        flag-inter-group-formal
      ]
    ]
    
    ;; Inter-group informal
    ;; If agents from at least 2 different groups are selected to interact, then this meeting type happens
    flag-inter-group-informal
  ]
  
  ;; Intra-group interactions
  
  ;; Intra-group formal
  ;; Formal meeting of all advisors and point persons in a group
  flag-intra-group-formal
  
  ;; Intra-group informal
  ;; Agents in the same group may interact
  flag-intra-group-informal
end


;; These next four procedures determines who will interact and then if conditions are right, then the interaction happens

to flag-inter-group-formal
  ;; Inter-group formal interaction
  ;; Interaction of all point persons
  ask point-persons [
    set updating? true
  ]
  set type-of-interaction "Intergroup Formal"
  set Count-inter-formal Count-inter-formal + 1
  interact
end



to flag-inter-group-informal
  ;; Inter-group informal interaction
  ;; Meetings setup based on information from the input files
  ;; The file is a list of pairings of agents in different groups and the probability that the interaction will happen each tick
  
  let meeting-groupings ( length inter-group-informal-list ) / 3 ;; number of these meetings potentially
  let meeting-groupings-happened 0
  let meeting-probability-list lput 999 inter-group-informal-list ;; Adding the 999 to the begining so the syntax of the list works
  let interacting-now []
  ;;; Repeat as many times as there are separate groupings of this meeting type
  repeat meeting-groupings [
    ;; Thance of this meeting taken from the list    
    let chance-interaction-from-list item ( ( meeting-groupings-happened * 3 ) + 2 ) meeting-probability-list
    ;; Draw random number
    let chance-intergroup-informal ( ( random 100 ) + 1 )
    if chance-intergroup-informal <= chance-interaction-from-list [
      ;; The meeting happens.
      ;; Make a list of the two interacting agents and then mark them as updating.
      set interacting-now sublist meeting-probability-list ( meeting-groupings-happened * 3 ) ( ( meeting-groupings-happened * 3 ) + 2 )
      ask turtles [
        if member? who interacting-now = true [
          set updating? true
        ]
      ]
      set type-of-interaction "Intergroup Informal"
      set Count-inter-informal Count-inter-informal + 1
      ;; Have the two agents interact
      interact
    ]
    set meeting-groupings-happened meeting-groupings-happened + 1     
  ]
end
  
to flag-intra-group-formal
  ;; Intragroup formal
  ;; Frequency specific to each case and each subgroup
  
  ;; Case 1
  if Scenario = "1a" or Scenario = "1b" or Scenario = "1c" or Scenario = "1d" [
    ;; Meetings happen every 3 months with group 0
    if ticks mod 64 = 0 [
      ask turtles with [ group = 0 ] [ set updating? true ]
      set type-of-interaction "Intragroup Formal"
      set Count-intra-formal Count-intra-formal + 1
      interact
    ]
    ;; Meetings happen daily with group 1
    if ticks mod 1 = 0 [
      ask turtles with [ group = 1 ] [ set updating? true ]
      set type-of-interaction "Intragroup Formal"
      set Count-intra-formal Count-intra-formal + 1
      interact
    ]
  ]
  
  ;; Case 2
  if Scenario = "2a" or Scenario = "2b" or Scenario = "2c" or Scenario = "2d" or Scenario = "2e" [
    ;; Meetings happen every 30 days with group 3
    if ticks mod 30 = 0 [
      ask turtles with [ group = 3 ] [ set updating? true ]
      set type-of-interaction "Intragroup Formal"
      set Count-intra-formal Count-intra-formal + 1
      interact
    ]
  ]
end



to flag-intra-group-informal
  ;; Intragroup informal
  ;; Agents in the same group may interact
  ;; Input file defines how many intragroup informal meetings may happen each day and the probability of each agent in that group being included in the interaction
  
  let meeting-informal-list lput 999 intra-group-informal-list
  let meeting-list-placeholder 0
  let intra-informal-group-counter1 0
  repeat Total-Number-of-Groups [
    ;; Repeat as many times as there are groups
    ;; Number of meetings per tick this groups has imported from list
    let number-intra-informal-meetings item meeting-list-placeholder meeting-informal-list
    ;; The number is not necessarily an interger and may be more than 1, so this converts any fraction into something meaningfil
    let automatic-meetings-happening int number-intra-informal-meetings ;; Automatic meetings = intergers
    let possible-meeting-chance ( number-intra-informal-meetings - automatic-meetings-happening ) * 100 ;; possible meetings is the fractional amount
    ;; Calculate how many meetings happening this tick with this group
    let meetings-happening-now 0
    if possible-meeting-chance != 0 [
      let intra-informal1 ( ( random 100 ) + 1 )
      if intra-informal1 <= possible-meeting-chance [
        set meetings-happening-now 1
      ]
    ]
    ;; Final number of meetings
    set meetings-happening-now meetings-happening-now + automatic-meetings-happening
    let number-in-this-group count turtles with [ group = intra-informal-group-counter1 ]    
    repeat meetings-happening-now [
      ;; Repeat as many times as the meeting happens
      let turtle-counter 0
      repeat number-in-this-group [
        ask turtle ( meeting-list-placeholder - intra-informal-group-counter1 + turtle-counter ) [
          ;; Go through the agents in the group and using the imported probability of the agent being included, determine if this agent is selected
          let turtle-selected-intra-informal ( ( random 100 ) + 1 )
          let chance-intra-group-informal-selection item ( meeting-list-placeholder + 1 + turtle-counter ) meeting-informal-list
          if turtle-selected-intra-informal <= chance-intra-group-informal-selection [ set updating? true ]
        ]
        set turtle-counter turtle-counter + 1
      ]
      ifelse count turtles with [ updating? = true ] > 1 [
        ;; As long as 2 were selected, have them interact, otherwise reset the selected agent
        set type-of-interaction "Intragroup Informal"
        set Count-intra-informal Count-intra-informal + 1
        interact
      ]
      [
        set sum-flagged ( sum-flagged + ( count turtles with [ updating? = true ] ) )
        ask turtles with [ updating? = true ] [ set updating? false ]
      ]
    ]
    set intra-informal-group-counter1 intra-informal-group-counter1 + 1
    set meeting-list-placeholder meeting-list-placeholder + 1 + number-in-this-group
  ]
end

to interact
  ;; What happens during an interaction.
  ;; First, update tracking variables
  set total-number-of-interactions total-number-of-interactions + 1
  ask turtles with [ updating? = true ] [ set number-of-times-interacting number-of-times-interacting + 1 ]
  set sum-number-of-times-interacting sum [ number-of-times-interacting ] of turtles
  set average-interactions-per-agent (sum-number-of-times-interacting / Total-Number-of-Agents)
  set sum-flagged ( sum-flagged + ( count turtles with [ updating? = true ] ) )
  
  ;; next, normalize respect values for agents that are part of interaction
  normalize-respect
  
  ;; Update the colors and dispplay to show agents involved and the normalized respect values
  do-colors
  update-display
  wait wait-time ;; Wait as long as a slider indicates, which makes it possible to monitor interactions
  
  ;; Update individual-positions weighted by respect
  update-position
  ;; Show interacting agents and patches differently to make them stand out and update their labels
  do-colors
  update-display
  ;; Reset updating flags and normalized respect variables
  reset
  ;; Reset the colors and labels of patches and agents
  do-colors
  update-display
end

to normalize-respect
  ;; This procedure allows to normalize the respect values in setup
  ;; In order for the math to work properly with the matrix multiplication,
  ;; the respect scores each agent has for the other agents with which it is interacting and itself need to add up to 1, so this normalized the respect values for the interacting agents only
  
  ask turtles with [ updating? = true ] [
    let respect-sum 0    
    ;; Asking patches corresponding to the agents interacting to add their respect value to the respect-sum
    ask patches with [ pycor = [ycor] of myself ] [
      if any? turtles with [ id = [ pxcor ] of myself and updating? = true] [ set respect-sum ( respect-sum + respect ) ]
    ]
    ;; Now, the respects are normalized by the total respect
    ask patches with [ pycor = [ycor] of myself ] [
      if any? turtles with [ id = [ pxcor ] of myself and updating? = true] [
        set normalized-respect ( respect / respect-sum )
      ]
    ]
  ]
end

to update-position
  ;; This procedure updates the individual-position for a conservation practice
  ;; agents' individual-position values are updated by calculating an average of the individual-positions of all agents it is interacting with including itself
  ;; weighted by the respect that each agent has for each other

  ;; Asking turtles that are updating
  ask turtles with [ updating? = true ] [
    ;; Initialize local variables
    let weighted-position 0
    let my-ycor ycor
    ;; Each turtle needs to ask each patch in its row (defined by the ycor), to multiply its stored value of respect by
    ;; the individual-position value stored by the turtle whose ID is defined by the xcor of the patch
    ask patches with [ pycor = my-ycor and normalized-respect != 0 ] [
      ;; Initializing more local variables
      let pxcor-here abs ( pxcor )
      let position-multiplier 0
      ;; Asking the turtle that is in the row with the patch to set variables (to store its individual-position)
      ask turtles with [ ID = pxcor-here ] [
        set position-multiplier individual-position
      ]
      ;;Calculating the new individual-position
      set weighted-position ( weighted-position + ( normalized-respect * position-multiplier ) )
    ]
    ;; Storing the final weighted-position in the buffer layer
    set position-future weighted-position
  ]
  ;; Setting the individual-position as the position-future-future after all calculations of position-future are done
  ask turtles with [ updating? = true ] [
    set individual-position position-future
  ]
end

to reset
  ;; Reset updating flags and normalized respect variables
  ask turtles with [ updating? = true ] [
    set updating? false
  ]
  ask patches with [ normalized-respect != 0 ] [
    set normalized-respect 0
  ]
end

to update-display
  ;; Labels for agents and patches. There are qualifiers for the ask patches since this is not being called by the observer in some cases.
  ask patches with [ normalized-respect != 0 ] [ set plabel precision normalized-respect 3 ]
  ask patches with [ normalized-respect = 0 ] [ set plabel precision normalized-respect 3 ]
  ask turtles with [ updating? = true ] [ set label precision individual-position 3 ]
  ask turtles with [ updating? = false ] [ set label precision individual-position 3 ]
end


to do-plots-and-globals
  ;; Setting this display-related variable to be blank, since there are no interactions happening.
  set type-of-interaction " "
  ;; Tracking metrics
  set collective-position mean [ individual-position ] of turtles
  set Spread-of-individual-positions standard-deviation [ individual-position ] of turtles 
  
  ;; Graphs
  
  set-current-plot "All Individual Positions Colored by Group"
  let turtle-count 0
  foreach sort turtles [
    ask ? [
      set-current-plot-pen word "" turtle-count
      
      set-plot-pen-color inactive-color - 2
      plot individual-position
      set turtle-count turtle-count + 1
    ]
  ]
    
  set-current-plot "Individual Position Histogram"
  set-current-plot-pen "default"
  set-plot-pen-mode 1
  set-histogram-num-bars 5
  histogram [ individual-position ] of turtles

  
  

  
  let DM-count1 0
  set-current-plot "Collective Position"
  set-current-plot-pen "Overall"
  plot collective-position
  foreach sort point-persons [
    ask ? [
      set-current-plot-pen word "Group " DM-count1
      plot ( mean [ individual-position ] of turtles with [ group = [ group ] of myself ] )
      set DM-count1 DM-count1 + 1
    ]
  ]    
  
  let DM-count 0
  set-current-plot "Spread of Individual Positions"
  set-current-plot-pen "Overall" 
  plot Spread-of-individual-positions
  foreach sort point-persons [
    ask ? [
      set-current-plot-pen word "Group " DM-count
      if count  turtles with [ group = [ group ] of myself ] > 1 [
        plot ( standard-deviation [ individual-position ] of turtles with [ group = [ group ] of myself ] )
      ]
      set DM-count DM-count + 1
    ]
  ]
 
end
@#$#@#$#@
GRAPHICS-WINDOW
569
10
1219
681
-1
-1
32.0
1
10
1
1
1
0
0
0
1
0
19
-19
0
0
0
1
ticks
30.0

BUTTON
437
159
500
192
NIL
Go
NIL
1
T
OBSERVER
NIL
NIL
NIL
NIL
1

BUTTON
437
120
501
153
NIL
Setup
NIL
1
T
OBSERVER
NIL
NIL
NIL
NIL
1

BUTTON
437
197
531
230
Go Forever
go
T
1
T
OBSERVER
NIL
NIL
NIL
NIL
1

PLOT
32
857
299
1130
All Individual Positions Colored by Group
Time
Position
0.0
64.0
0.0
1.0
true
false
"" ""
PENS
"0" 1.0 0 -16777216 true "" ""
"1" 1.0 0 -16777216 true "" ""
"2" 1.0 0 -16777216 true "" ""
"3" 1.0 0 -16777216 true "" ""
"4" 1.0 0 -16777216 true "" ""
"5" 1.0 0 -16777216 true "" ""
"6" 1.0 0 -16777216 true "" ""
"7" 1.0 0 -16777216 true "" ""
"8" 1.0 0 -16777216 true "" ""
"9" 1.0 0 -16777216 true "" ""
"10" 1.0 0 -16777216 true "" ""
"11" 1.0 0 -16777216 true "" ""
"12" 1.0 0 -16777216 true "" ""
"13" 1.0 0 -16777216 true "" ""
"14" 1.0 0 -16777216 true "" ""
"15" 1.0 0 -16777216 true "" ""
"16" 1.0 0 -16777216 true "" ""
"17" 1.0 0 -16777216 true "" ""
"18" 1.0 0 -16777216 true "" ""
"19" 1.0 0 -16777216 true "" ""
"20" 1.0 0 -16777216 true "" ""
"21" 1.0 0 -16777216 true "" ""
"22" 1.0 0 -16777216 true "" ""
"23" 1.0 0 -16777216 true "" ""
"24" 1.0 0 -16777216 true "" ""
"25" 1.0 0 -16777216 true "" ""
"26" 1.0 0 -16777216 true "" ""
"27" 1.0 0 -16777216 true "" ""

PLOT
311
859
510
986
Individual Position Histogram
Position Value
Occurances
0.0
1.001
0.0
1.0
true
false
"" ""
PENS
"default" 1.0 0 -16777216 true "" ""

MONITOR
25
284
192
345
Collective Position
precision collective-position 3
4
1
15

SLIDER
166
130
338
163
Wait-time
Wait-time
0
5
0
.5
1
Seconds
HORIZONTAL

PLOT
32
703
354
853
Spread of Individual Positions
Time
Spread
0.0
64.0
0.0
0.4
true
true
"" ""
PENS
"Overall" 1.0 0 -16777216 true "" ""
"Group 0" 1.0 0 -13791810 true "" ""
"Group 1" 1.0 0 -5825686 true "" ""
"Group 2" 1.0 0 -955883 true "" ""
"Group 3" 1.0 0 -1184463 true "" ""

PLOT
32
544
354
700
Collective Position
Time
Position
0.0
64.0
0.0
1.0
true
true
"" ""
PENS
"Overall" 1.0 0 -16777216 true "" ""
"Group 0" 1.0 0 -13791810 true "" ""
"Group 1" 1.0 0 -5825686 true "" ""
"Group 2" 1.0 0 -955883 true "" ""
"Group 3" 1.0 0 -1184463 true "" ""

TEXTBOX
28
261
178
282
Primary Metrics
15
0.0
1

MONITOR
198
284
430
345
Spread of Iindividual Positions
Spread-of-individual-positions
4
1
15

TEXTBOX
27
362
177
381
Meeting Counts
15
0.0
1

MONITOR
166
168
406
229
Type of Interaction Occuring Now
type-of-interaction
17
1
15

TEXTBOX
218
234
422
262
Interacting agents highlighted at right
11
0.0
1

TEXTBOX
37
510
187
529
Metrics Over Time
15
0.0
1

CHOOSER
17
62
534
107
Scenario-Tested
Scenario-Tested
"1a - No KL - Baseline" "1b - With KL - high respect and high interactions" "1c - With KL - high respect and low interactions" "1d- With KL - low respect and high interactions" "2a - No KL – baseline" "2b - With KL - high respect and high interactions" "2c - With KL - high respect and high interactions but KL has low respect for Subgroup 3 agents" "2d - With KL - high respect and high interactions - Subgroup 0 and 3 same size" "2e - No KL - Subgroup 0 point person replaced with agent that interacts more and has higher respect"
6

MONITOR
23
435
230
480
Count of Inter-group Informal Meetings
Count-inter-informal
17
1
11

MONITOR
23
386
231
431
Count of Inter-group Formal Meetings
Count-inter-formal
17
1
11

MONITOR
240
386
448
431
Count of Intra-group Formal Meetings
Count-intra-formal
17
1
11

MONITOR
241
436
448
481
Count Iintra-group Informal Meetings
Count-intra-informal
17
1
11

MONITOR
437
284
518
345
Final Tick
final-tick
0
1
15

@#$#@#$#@
##COPYRIGHT NOTICE

CoDMER Version 2.0 December 2013

(C)2013 Board of Trustees of the University of Illinois
    
Authors: 
Moira Zellner, Dean Massey and Kristen Ross, University of Illinois at Chicago
Cristy Watkins, Field Museum
Lynne Westphal, US Forest Service
Jeremy Brooks, Ohio State University

Please refer to the overview of the model that is available to download along with this NetLogo file at www.openabm.org to see the full table and figures referred to in the text below. Also make sure you download the accompanying input files.

##PURPOSE

Ecological restoration, particularly in urban contexts, is a complex collective decision-making process that involves a diversity of stakeholders and experts, each with their own perceptions about what landscapes should and can look like, how to get them to the desired state, and on what timeline. CoDMER can be used to investigate how structural and behavioral factors can influence collective decision making processes and outcomes in the context of ecological restoration. It is informed by existing literature on collective decision making and empirical data from the Chicago Wilderness region. This new version 2.0 has been parameterized with ethnographic data from two Chicago Wilderness member organizations dealing making decisions about prescribed fire (Case 1) and seeding native plants (Case 2), to advance theory on how collective decisions emerge in this context.

For more details, please refer to Zellner, M. L.; Watkins, C.; Massey, D.; Westphal, L. M.; Brooks, J.; Ross, K. (in review). “Advancing collective decision-making theory with integrated agent-based modeling and ethnographic data analysis: An example in ecological restoration.” Journal of Artificial Societies and Social Simulations.

##HOW IT WORKS

Prior to initialization, define a scenario by setting the chooser to the corresponding scenario. All scenarios corresponding to Case 1 start with 1, all those corresponding to Case 2, with 2. A description of these cases are in the article cited above.

Press Setup to load the corresponding input parameters, and Go to run the simulation. 
The model represents one planning season for the ecological restoration organization. One iteration represents one work day, during which agents (advisors and point persons) interact with selected others through formal and informal meetings, and update their position with respect to a restoration decision based on how they are influenced by the other agents. The run ends when individual positions are within 0.01 of each other, and the collective position and length of run are recorded.

Every iteration, the model checks for four different types of possible interactions at defined probabilities and frequencies for each scenario:
(1) formal intergroup interactions at regular intervals involving the point persons of each subgroup; 
(2) formal intra-group interactions occurring at regular intervals and involving all the advisors and the point-person from a single group; 
(3) informal inter-group interactions involving a specific pairing of agents from two different subgroups, with each pairing assigned a daily probability of meeting, and 
(4) informal intra-group interactions involving at least two agents in the same group; the meeting occurs at a specified daily probability (a maximum of 1 meeting per day), and the agents involved are selected according to individual probabilities..

If, during the check for interactions procedure, the conditions for an interaction are met, the respect values that agents have for themselves and for all the other agents involved in the specific interaction are normalized to ensure relative weighting is only applied across the values of agents involved in the interaction, and not all the agents in the organization.

After respect values are normalized, each agent’s position value is then updated by calculating an average of the positions of all interacting agents, weighted by the respect that each agent has for each other.

##THINGS TO TRY

Examine each scenario to understand the effect of a liasion agent (KL), the distribution of respect across groups, group size, and the structure of interactions.

##HOW TO CITE

Zellner, M. L.; Massey, D.; Watkins, C.; Westphal, L. M.; Ross, K.; Brooks, J. (2013). CoDMER v. 2.0: Collective Decision Making for Ecological Restoration version 2.0. Department of Urban Planning and Policy. University of Illinois at Chicago.

##REFERENCES

Zellner, M. L.; Watkins, C.; Massey, D.; Westphal, L. M.; Brooks, J.; Ross, K. (in review). “Advancing collective decision-making theory with integrated agent-based modeling and ethnographic data analysis: An example in ecological restoration.” Journal of Artificial Societies and Social Simulations.
@#$#@#$#@
default
true
0
Polygon -7500403 true true 150 5 40 250 150 205 260 250

airplane
true
0
Polygon -7500403 true true 150 0 135 15 120 60 120 105 15 165 15 195 120 180 135 240 105 270 120 285 150 270 180 285 210 270 165 240 180 180 285 195 285 165 180 105 180 60 165 15

arrow
true
0
Polygon -7500403 true true 150 0 0 150 105 150 105 293 195 293 195 150 300 150

box
false
0
Polygon -7500403 true true 150 285 285 225 285 75 150 135
Polygon -7500403 true true 150 135 15 75 150 15 285 75
Polygon -7500403 true true 15 75 15 225 150 285 150 135
Line -16777216 false 150 285 150 135
Line -16777216 false 150 135 15 75
Line -16777216 false 150 135 285 75

bug
true
0
Circle -7500403 true true 96 182 108
Circle -7500403 true true 110 127 80
Circle -7500403 true true 110 75 80
Line -7500403 true 150 100 80 30
Line -7500403 true 150 100 220 30

butterfly
true
0
Polygon -7500403 true true 150 165 209 199 225 225 225 255 195 270 165 255 150 240
Polygon -7500403 true true 150 165 89 198 75 225 75 255 105 270 135 255 150 240
Polygon -7500403 true true 139 148 100 105 55 90 25 90 10 105 10 135 25 180 40 195 85 194 139 163
Polygon -7500403 true true 162 150 200 105 245 90 275 90 290 105 290 135 275 180 260 195 215 195 162 165
Polygon -16777216 true false 150 255 135 225 120 150 135 120 150 105 165 120 180 150 165 225
Circle -16777216 true false 135 90 30
Line -16777216 false 150 105 195 60
Line -16777216 false 150 105 105 60

car
false
0
Polygon -7500403 true true 300 180 279 164 261 144 240 135 226 132 213 106 203 84 185 63 159 50 135 50 75 60 0 150 0 165 0 225 300 225 300 180
Circle -16777216 true false 180 180 90
Circle -16777216 true false 30 180 90
Polygon -16777216 true false 162 80 132 78 134 135 209 135 194 105 189 96 180 89
Circle -7500403 true true 47 195 58
Circle -7500403 true true 195 195 58

circle
false
0
Circle -7500403 true true 0 0 300

circle 2
false
0
Circle -7500403 true true 0 0 300
Circle -16777216 true false 30 30 240

cow
false
0
Polygon -7500403 true true 200 193 197 249 179 249 177 196 166 187 140 189 93 191 78 179 72 211 49 209 48 181 37 149 25 120 25 89 45 72 103 84 179 75 198 76 252 64 272 81 293 103 285 121 255 121 242 118 224 167
Polygon -7500403 true true 73 210 86 251 62 249 48 208
Polygon -7500403 true true 25 114 16 195 9 204 23 213 25 200 39 123

cylinder
false
0
Circle -7500403 true true 0 0 300

dot
false
0
Circle -7500403 true true 90 90 120

face happy
false
0
Circle -7500403 true true 8 8 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Polygon -16777216 true false 150 255 90 239 62 213 47 191 67 179 90 203 109 218 150 225 192 218 210 203 227 181 251 194 236 217 212 240

face neutral
false
0
Circle -7500403 true true 8 7 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Rectangle -16777216 true false 60 195 240 225

face sad
false
0
Circle -7500403 true true 8 8 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Polygon -16777216 true false 150 168 90 184 62 210 47 232 67 244 90 220 109 205 150 198 192 205 210 220 227 242 251 229 236 206 212 183

fish
false
0
Polygon -1 true false 44 131 21 87 15 86 0 120 15 150 0 180 13 214 20 212 45 166
Polygon -1 true false 135 195 119 235 95 218 76 210 46 204 60 165
Polygon -1 true false 75 45 83 77 71 103 86 114 166 78 135 60
Polygon -7500403 true true 30 136 151 77 226 81 280 119 292 146 292 160 287 170 270 195 195 210 151 212 30 166
Circle -16777216 true false 215 106 30

flag
false
0
Rectangle -7500403 true true 60 15 75 300
Polygon -7500403 true true 90 150 270 90 90 30
Line -7500403 true 75 135 90 135
Line -7500403 true 75 45 90 45

flower
false
0
Polygon -10899396 true false 135 120 165 165 180 210 180 240 150 300 165 300 195 240 195 195 165 135
Circle -7500403 true true 85 132 38
Circle -7500403 true true 130 147 38
Circle -7500403 true true 192 85 38
Circle -7500403 true true 85 40 38
Circle -7500403 true true 177 40 38
Circle -7500403 true true 177 132 38
Circle -7500403 true true 70 85 38
Circle -7500403 true true 130 25 38
Circle -7500403 true true 96 51 108
Circle -16777216 true false 113 68 74
Polygon -10899396 true false 189 233 219 188 249 173 279 188 234 218
Polygon -10899396 true false 180 255 150 210 105 210 75 240 135 240

house
false
0
Rectangle -7500403 true true 45 120 255 285
Rectangle -16777216 true false 120 210 180 285
Polygon -7500403 true true 15 120 150 15 285 120
Line -16777216 false 30 120 270 120

leaf
false
0
Polygon -7500403 true true 150 210 135 195 120 210 60 210 30 195 60 180 60 165 15 135 30 120 15 105 40 104 45 90 60 90 90 105 105 120 120 120 105 60 120 60 135 30 150 15 165 30 180 60 195 60 180 120 195 120 210 105 240 90 255 90 263 104 285 105 270 120 285 135 240 165 240 180 270 195 240 210 180 210 165 195
Polygon -7500403 true true 135 195 135 240 120 255 105 255 105 285 135 285 165 240 165 195

line
true
0
Line -7500403 true 150 0 150 300

line half
true
0
Line -7500403 true 150 0 150 150

pentagon
false
0
Polygon -7500403 true true 150 15 15 120 60 285 240 285 285 120

person
false
0
Circle -7500403 true true 110 5 80
Polygon -7500403 true true 105 90 120 195 90 285 105 300 135 300 150 225 165 300 195 300 210 285 180 195 195 90
Rectangle -7500403 true true 127 79 172 94
Polygon -7500403 true true 195 90 240 150 225 180 165 105
Polygon -7500403 true true 105 90 60 150 75 180 135 105

plant
false
0
Rectangle -7500403 true true 135 90 165 300
Polygon -7500403 true true 135 255 90 210 45 195 75 255 135 285
Polygon -7500403 true true 165 255 210 210 255 195 225 255 165 285
Polygon -7500403 true true 135 180 90 135 45 120 75 180 135 210
Polygon -7500403 true true 165 180 165 210 225 180 255 120 210 135
Polygon -7500403 true true 135 105 90 60 45 45 75 105 135 135
Polygon -7500403 true true 165 105 165 135 225 105 255 45 210 60
Polygon -7500403 true true 135 90 120 45 150 15 180 45 165 90

sheep
false
0
Rectangle -7500403 true true 151 225 180 285
Rectangle -7500403 true true 47 225 75 285
Rectangle -7500403 true true 15 75 210 225
Circle -7500403 true true 135 75 150
Circle -16777216 true false 165 76 116

square
false
0
Rectangle -7500403 true true 30 30 270 270

square 2
false
0
Rectangle -7500403 true true 30 30 270 270
Rectangle -16777216 true false 60 60 240 240

star
false
0
Polygon -7500403 true true 151 1 185 108 298 108 207 175 242 282 151 216 59 282 94 175 3 108 116 108

target
false
0
Circle -7500403 true true 0 0 300
Circle -16777216 true false 30 30 240
Circle -7500403 true true 60 60 180
Circle -16777216 true false 90 90 120
Circle -7500403 true true 120 120 60

tree
false
0
Circle -7500403 true true 118 3 94
Rectangle -6459832 true false 120 195 180 300
Circle -7500403 true true 65 21 108
Circle -7500403 true true 116 41 127
Circle -7500403 true true 45 90 120
Circle -7500403 true true 104 74 152

triangle
false
0
Polygon -7500403 true true 150 30 15 255 285 255

triangle 2
false
0
Polygon -7500403 true true 150 30 15 255 285 255
Polygon -16777216 true false 151 99 225 223 75 224

truck
false
0
Rectangle -7500403 true true 4 45 195 187
Polygon -7500403 true true 296 193 296 150 259 134 244 104 208 104 207 194
Rectangle -1 true false 195 60 195 105
Polygon -16777216 true false 238 112 252 141 219 141 218 112
Circle -16777216 true false 234 174 42
Rectangle -7500403 true true 181 185 214 194
Circle -16777216 true false 144 174 42
Circle -16777216 true false 24 174 42
Circle -7500403 false true 24 174 42
Circle -7500403 false true 144 174 42
Circle -7500403 false true 234 174 42

turtle
true
0
Polygon -10899396 true false 215 204 240 233 246 254 228 266 215 252 193 210
Polygon -10899396 true false 195 90 225 75 245 75 260 89 269 108 261 124 240 105 225 105 210 105
Polygon -10899396 true false 105 90 75 75 55 75 40 89 31 108 39 124 60 105 75 105 90 105
Polygon -10899396 true false 132 85 134 64 107 51 108 17 150 2 192 18 192 52 169 65 172 87
Polygon -10899396 true false 85 204 60 233 54 254 72 266 85 252 107 210
Polygon -7500403 true true 119 75 179 75 209 101 224 135 220 225 175 261 128 261 81 224 74 135 88 99

wheel
false
0
Circle -7500403 true true 3 3 294
Circle -16777216 true false 30 30 240
Line -7500403 true 150 285 150 15
Line -7500403 true 15 150 285 150
Circle -7500403 true true 120 120 60
Line -7500403 true 216 40 79 269
Line -7500403 true 40 84 269 221
Line -7500403 true 40 216 269 79
Line -7500403 true 84 40 221 269

x
false
0
Polygon -7500403 true true 270 75 225 30 30 225 75 270
Polygon -7500403 true true 30 75 75 30 270 225 225 270

@#$#@#$#@
NetLogo 5.0.2
@#$#@#$#@
@#$#@#$#@
@#$#@#$#@
@#$#@#$#@
@#$#@#$#@
default
0.0
-0.2 0 1.0 0.0
0.0 1 1.0 0.0
0.2 0 1.0 0.0
link direction
true
0
Line -7500403 true 150 150 90 180
Line -7500403 true 150 150 210 180

@#$#@#$#@
0
@#$#@#$#@
